
const Webpack = require('webpack');
const WebpackNotifierPlugin = require('webpack-notifier');
const path = require('path');
const fs = require('fs');

// const appPath = path.resolve(__dirname, 'app');
// const buildPath = path.resolve(__dirname, 'public', 'build');
// const nodeModulesPath = path.resolve(__dirname, 'node_modules');


function isDirectory(dir) {
    return fs.lstatSync(dir).isDirectory();
}

module.exports =
{
    devtool: 'inline-source-map',

    entry: fs.readdirSync(__dirname).reduce(function r(entries, dir) {
        const isDraft = dir.charAt(0) === '_';

        if (!isDraft && isDirectory(path.join(__dirname, dir))) {
            entries[dir] = path.join(__dirname, dir, 'app.js');
        }

        return entries;
    }, {}),

    output: {
        path: 'examples/__build__',
        filename: '[name].js',
        chunkFilename: '[id].chunk.js',
        publicPath: '/__build__/',
    },

    // cache: true,
    // debug: true,
    // context: path.join(__dirname, 'app'),

    module: {
        loaders: [
            {
                test: /\.(js|jsx)$/,
                loader: 'babel?optional[]=runtime&stage=0',
                exclude: /(node_modules|bower_components)/,
            },
        ],
    },

    resolve: {
        alias: {
            'formless': '../../src',
        },
    },

    plugins: [
        new WebpackNotifierPlugin(),
        new Webpack.optimize.CommonsChunkPlugin('shared.js'),
        new Webpack.DefinePlugin({
            'process.env.NODE_ENV': JSON.stringify(process.env.NODE_ENV || 'development'),
        }),
    ],

    noInfo: true, //  --no-info option
    
    // devServer: {
    //     headers: { 'Access-Control-Allow-Origin': '*' },
    //     contentBase: './build',
    //     noInfo: true, //  --no-info option
    //     hot: true,
    //     inline: true,
    //     port: 8090,
    // },
};
