import React from 'react/addons';
const PureRenderMixin = React.addons.PureRenderMixin;
import classNames from 'classnames';

class BaseContainer extends React.Component {

    constructor(props) {
        super(props);
        this.shouldComponentUpdate = PureRenderMixin.shouldComponentUpdate.bind(this);
        this.state = {showError: null, showWarn: null, errors: null};
    }


    renderLabel(extraClasses) {
        if (!this.props.label) {
            return '';
        }

        const labelClasses = classNames({
            // 'control-label': true,
            [extraClasses]: true,
            [this.props.labelClasses]: !!this.props.labelClasses,
        });

        return (
            <label className={labelClasses}>
                { this.props.label }
                { this.props.isRequired ?
                    <span className='required'>*</span> : '' }
            </label>
        );
    }

    renderError() {
        if (!this.props.errorMsg) {
            return '';
        }

        let errors = [];

        // Convert to array.
        if (typeof this.props.errorMsg === 'object') {
            errors = Object.keys(this.props.errorMsg).map(k => this.props.errorMsg[k]);
        } else {
            errors = [this.props.errorMsg];
        }

        return (
            <ul className="parsley-errors-list filled">
                {
                    errors.map((e, i) => {
                        return ( <li key={ i }>{ e.toString() }</li>);
                    })
                }
            </ul>
        );
    }

    _hasError() {
        if (!this.props.errorMsg) {
            return false;
        }

        if (typeof this.props.errorMsg === 'object') {
            return this.props.errorMsg.length > 0;
        }

        return !!this.props.errorMsg;
    }

    renderDescription() {
        if (!this.props.description) {
            return '';
        }

        return (
            <p className='help-block'>
                <small
                    dangerouslySetInnerHTML={{__html: this.props.description}}/>
            </p>
        );
    }

    render() {
        const outerClasses = classNames({
            'form-group': true,
            // 'has-error': this._hasError(),
            [this.props.outerClasses]: !!this.props.outerClasses,
        });

        return (
            <div className={outerClasses}>

                { this.renderLabel() }
                { this.props.children }
                { this.renderError() }
                { this.renderDescription() }

            </div>
        );
    }
}

BaseContainer.propTypes = {
    type: React.PropTypes.string,
    label: React.PropTypes.string,
    description: React.PropTypes.string,
    outerClasses: React.PropTypes.string,
    labelClasses: React.PropTypes.string,
    errorMsg: React.PropTypes.oneOfType([
        React.PropTypes.string,
        React.PropTypes.array,
    ]),
    isRequired: React.PropTypes.bool,
};

export default BaseContainer;
