
import React from 'react/addons';
import Immutable from 'immutable';
import objectPath from 'object-path';
const PureRenderMixin = React.addons.PureRenderMixin;


// TODO: also keypath...
class FormController extends React.Component {

    constructor(props) {
        super(props);
        this.shouldComponentUpdate = PureRenderMixin.shouldComponentUpdate.bind(this);

        this.state = Immutable.fromJS({
            isValid: true,
            isValidating: false,
            errors: {}, // TODO: should I be in the widgets definition or internal to them.
            serverErrors: {}, // TODO: needs to be populated and displayed.
            hidden: {}, // The fields that have been hidden.
            disabled: {},
        });

        this._formData = Immutable.fromJS({
            isPristene: true, // Has any of the widgets been touched, or the form submitted.
            values: {}, // The values of the widgets.
            widgets: {},
            hasSubmitted: false, // Has the form been submitted.
            // touchedFields: {},
        });
    }

    // Set's the form to say validation has started, also set's the isValid to false.
    _setIsValidating(isValidating) {
        let newState = {isValidating};
        if (isValidating) {
           newState.isValid = false;
        }
        this.setState(newState);
    }

    _getIsValidating() {
        return this.state.isValidating;
    }

    // TODO: validation failed to complete?
    onFormSubmit(e) {
		e.preventDefault();

        this._setIsValidating(true);

        let validations = [];

        // Get all the promises for validating each widget.
        this._formData.get('widgets')
        .map( widget => {
            const q = widget._onSubmit()
            .then( (res) => {
                return res;
            });

            validations.push(q);
        }).toArray();

        // run all the validations
        Promise.all(validations)
        .then( (data) => {
            this._setIsValidating(false);

            // doing this inverse so it stops at the first invalid.
            const isInValid = data.some( d => !d.isValid() );
            
            this.setState({
                hasSubmitted: true,
                isPristene: false,
                isValid: !isInValid,
            });

            if (!isInValid) {
                // If we are embedding the form.
                if (this.props.onValidSubmit) {
                    this.props.onValidSubmit(this._getValues());
                }

                // If we are directly sub-classing and we want the result.
                if (this._onValidSubmit) {
                    this._onValidSubmit(this._getValues());
                }
            }

        })
        .catch( (err) => {
            console.log(err, '<<<<<<< FORM validation failed');
        });
    }

    _onInputUpdate(w, result) {
        const name = w.props.name;

        if (result.isValid()) {
            this._formData = this._formData.setIn(['values'].concat(name.split('.')), result.value);
        }

        this._setNotPristene();
    }

	_setNotPristene() {
        this._formData = this._formData.set('isPristene', false);
	}

    _getValues() {
        return this._formData.get('values').toJS();
    }

    /* hideErrors= !this.state.hasSubmitted  */

    attatchToForm(w) {
        const name = w.props.name;
        this._formData = this._formData.setIn(['widgets', name], w);
    }

    detatchFromForm(w) {
        const name = w.props.name;
        this._formData = this._formData.deleteIn(['widgets', name], w);
        this._formData = this._formData.deleteIn(['values'].concat(name.split('.')), w);
    }

    traverseChildrenAndRegisterInputs(children) {
        if (typeof children !== 'object' || children === null) {
            return children;
        }

        return React.Children.map(children, (child) => {

            if (typeof child !== 'object' || child === null) {
                return child;
            }

            if (child.props && child.props.name) {

                return React.cloneElement(child, {
                    formController: this,
                }, child.props && child.props.children);
            }

            return React.cloneElement(child, {},
                this.traverseChildrenAndRegisterInputs(child.props && child.props.children));
        });
    }

}

//TODO: values:  initial values... get copied
FormController.propTypes = {
    onValidSubmit: React.PropTypes.func,
};

export default FormController;
