
import ValidationResult from './result.js';
import { validatorByName } from './lookup.js';
import { Required } from './validators';

// TODO: cancel validations?
// TODO: failed validations, server error 500

class Runner {

    constructor(opts) {
        this.label = opts.label;
        this.result = null;
        this.rawValue = null;
        this.isRequired = false;

        this.validators = this._createValidators(opts.validators);
    }

    setResult(result) {
        this.result = result;
    }

    getResult() {
        return this.result;
    }

    getIsRequired() {
        return this.isRequired;
    }

    _createValidators(validators) {
        if (!validators) {
            return null;
        }

        return validators.map( validatorDef => {
            const Val = validatorByName(validatorDef.name);
            if (Val === Required) {
                this.isRequired = true;
            }

            if (!validatorDef.label) {
                validatorDef.label = this.label;
            }

            return new Val(validatorDef);
        });
    }

    _shouldRunValidations(val) {
		return val !== this.rawValue || !this.result;
    }

    _shouldSkipValidator(validator, result) {
        return (validator.skipIfErrors && result.errors.length > 0);
    }

    recursionBaby(validators, result, resolve, reject) {

        if (!validators || validators.length === 0) {
            return resolve(result);
        }

        const validator = validators[0];

        // Should we skip this validator?
        if (this._shouldSkipValidator(validator, result)) {
            return this.recursionBaby(validators.slice(1), result, resolve, reject);
        }

        return validator.runValidation(result.value)
            .then( res => {
                // Modify our result object
                result.value = res.value;
                res.errors.map( e => result.addError(e) );

                // If we need to stop if we find any errors.
                if (validator.breakOnError && result.errors.length > 0) {
                    return this.recursionBaby([], result, resolve, reject);
                }

                const endValidators = validators.slice(1);
                this.recursionBaby(endValidators, result, resolve, reject);
            })
            .catch( err => reject(err) );
    }


    runValidations(value) {

        // If there is no validators to run, just save the value.
        if (!this.validators) {
            const result = new ValidationResult({value});
            this.setResult(result);
			return Promise.resolve(result);
        }

		// Nothing has changed and we have already run the validation.
		if ( !this._shouldRunValidations(value) ) {
			// Return the existing validation result.
			return Promise.resolve( this.result );
		}

        // Save this to see if we should run the validations later.
        this.rawValue = value;

        return new Promise( (resolve, reject) => {
            const result = new ValidationResult({value});
            this.recursionBaby(this.validators, result, resolve, reject);
        })
        .then( res => {
            // After the validations have run, save the result.
            this.setResult(res);
            return res;
        });
    }
}

export default Runner;
