
import { ErrorReport, ErrorCodes } from '../errorCodes.js';


/**
 * The base validation context is not intended to be instanciated
 * directly but inherited from.
 */
class BaseValidationContext {

    constructor(options) {
        this.skipIfErrors = options.skipIfErrors; // if any errors are found don't run this validaton.
        this.breakOnError = options.breakOnError; // once validation has run and an error is found don't run next validator.
        this.label = options.label; // The label to use in error msgs.
        this.msg = options.msg; // Over ride the default error msg.
    }

    // NOTE: renaming this from apply as it's a reserved js word that is very confusting
    runValidation(value) {
        throw new Error('runValidation must be over ridden:', value);
    }

    clientError(errorMsg) {
        // We can override it with a custom message.
        if (this.msg) {
            return new ErrorReport(ErrorCodes.client, this.msg);
        }

        return new ErrorReport(ErrorCodes.client, errorMsg);
    }

// Do not extend this directly, extend one from the next level up!.
}


export default BaseValidationContext;
