
import BaseValidationContext from './base.js';
import ValidationResult from '../result.js';

/**
* The base validation context.
*/
class BasicLimitedTextField extends BaseValidationContext {

    constructor(options) {
        super(options);

        this.notShorter = options.notShorter;
        this.notLonger = options.notLonger;
        this.allowEmpty = options.allowEmpty;
        this.trim = options.trim;
    }

    runValidation(toTest) {
        let result = new ValidationResult({value: toTest});

        // If we are null then drop out, it can be picked up by the required validator.
        if (result.value === null) {
            return Promise.resolve(result);
        }

        // Trim result first before processing.
        // if (this.trim !== null && this.time === true) {
        if (this.trim) {
            result.value = result.value.trim();
        }

        if (this.allowEmpty !== null) {
            if (result.value.length === 0) {
                result.addError(
                    this.clientError(`${this.label} is empty`)
                );
            }
        }

        if (this.notLonger !== null && result.value !== null && toTest.length > this.notLonger) {
            result.addError(
                this.clientError(`${this.label} is too long.`)
            );
        }

        if (this.notShorter !== null && result.value !== null && toTest.length < this.notShorter) {
            result.addError(
                this.clientError(`${this.label} is too short.`)
            );
        }

        return Promise.resolve(result);
    }
}

export default BasicLimitedTextField;
