
import BaseValidationContext from './base.js';
import ValidationResult from '../result.js';

/**
 * The base validation context.
 */
class RegexLimitedTextField extends BaseValidationContext {

    constructor(options) {
        super(options);
        this.regex = options.regex;

        if (typeof this.regex === 'string') {
            this.regex = new RegExp(this.regex);
        }

        if (this.regex.constructor.name !== 'RegExp') {
            throw new Error("'regex' parameter is not a string that can be compiled to a regex or it is not a RegExp object.");
        }

        this.invert = options.invert;
    }

    runValidation(toTest) {
        let result = new ValidationResult({value: toTest});

        let pr = this.regex.test(result.value);
        if ( self.invert ) {
            pr = !pr;
        }

        if (!pr) {
            if (!self.error) {
                result.addError(
                    this.clientError(`${this.label} is not valid.`)
                );
            }
        }

        return Promise.resolve(result);
    }
}

export default RegexLimitedTextField;
