
import BaseValidationContext from './base.js';
import ValidationResult from '../result.js';

/**
* The base validation context.
*/
class Required extends BaseValidationContext {

    constructor(options) {
        super(options);

        // trim by default.
        if (options.trim === undefined) {
            this.trim = true;
        } else {
            this.trim = options.trim;
        }

        // By default leave validation if this fails.
        if (options.breakOnError === undefined) {
            this.breakOnError = true;
        }
    }

    runValidation(toTest) {
        let result = new ValidationResult({value: toTest});

        // Trim result first before processing.
        if (result.value !== null && this.trim) {
            result.value = result.value.trim();
        }

        if (result.value === null || result.value.length === 0) {
            result.addError(
                this.clientError(`${this.label} is a required field.`)
            );
        }

        return Promise.resolve(result);
    }
}

export default Required;
