
import Containers from './containers';
import Widgets from './widgets';


function getDisplayWidget(type) {
	let widget;

	switch (type) {
        case 'tel':
		case 'text':
		case 'email':
		case 'conditionaltext':
			widget = Widgets.TextInput;
			break;
		case 'textarea':
			widget = Widgets.TextArea;
			break;
		case 'heading':
			widget = Widgets.Heading;
			break;
		case 'country':
		case 'dropdown':
			widget = Widgets.Dropdown;
			break;
		case 'checkbox_group':
			widget = Widgets.CheckboxSingle;
			break;
		case 'linebreak':
			widget = Widgets.Linebreak;
			break;
		case 'paragraph':
			widget = Widgets.Paragraph;
			break;
		case 'radio_group':
			widget = Widgets.RadioGroup;
			break;
		case 'button_group':
			widget = Widgets.ButtonGroup;
			break;
		case 'date':
			widget = Widgets.DateInput;
			break;
		case 'time':
			widget = Widgets.TimeInput;
			break;
		case 'payment_optional_item':
			widget = Widgets.PaymentOptional;
			break;
		case 'sponsorship':
			widget = Widgets.Sponsorship;
			break;
		case 'donation':
			widget = Widgets.Donation;
			break;
		case 'teams':
			widget = Widgets.Team;
			break;
		case 'cvv':
			widget = Widgets.Cvv;
			break;
		case 'terms': // TODO: special case!
		default:
			return null;
	}

	return widget;
}

function validationForWidget(w) {
	const vArray = w.get('Validation');
	if (!vArray) {
		return null;
	}

	return vArray.map( v => {
		if (v === 'required') {
			return {name: 'required'};
		}

		if (v === 'email') {
			return {name: 'email'};
		}

        if ( /^length/.test(v) ) {
            const [, min, max] = v.match(/^length\[(\d+)\,(\d+)\]/);
			return {name: 'basicText', notShorter: min, notLonger: max };
		}
	})
	.filter(v => !!v)
	.toJS();
}


function renderWidget(w, opts) {
    const name = w.get('Name');
    const type = w.get('Type');
    const Widget = getDisplayWidget(type);
    if (!Widget) {
        return null;
    }

    // A unique key for each viewer/widget so that react knows when to redraw.
    return (
        <Widget
            key={ 'widget_' + name }
            type={ type }
            name={ name }
            layout={ w.get('Layout') }
            label={ w.get('Label') }
            description={ w.get('Description') }
            placeholder={ w.get('Placeholder') }
			options={ w.get('Children') }
			validation={ validationForWidget(w) }
            outerClasses={ w.get('OuterClasses') }
            innerClasses={ w.get('InnerClasses') }
            labelClasses={ w.get('LabelClasses') }

			viewerIndex={ opts.viewerIndex /* only used for the team widget at this stage */ }
			onChangeFns={ opts.onChangeFns }
			formController={ opts.formController }
            onInputUpdate={ opts.onInputUpdate }
			value={ opts.value }

        />
    );
}
// validatorContext = {w.get('ValidationContext')}
// value={ this.state.values[name] }
// errorMsg={ this._errorMsgForWidget(w) }
// onInputUpdate={ this._onInputUpdate.bind(this, w) }

// TODO: container recursion?
function renderWidgets(widgets) {
    return widgets.map( (w) => {
        if (w.get('Type') === 'group') {

            // TODO: add default..
            const Container = Containers.getForLayout(w.get('Layout'));
                    // errorMsg={ this._errorMsgForContianer(w.get('Widgets')) }
                    // isRequired={ this._isRequiredForContainer(w.get('Widgets')) }
            return (
                <Container
                    key={ w.get('Name') }
                    label={ w.get('Label') }
                    description={ w.get('Description') }
                    outerClasses={ w.get('OuterClasses') }
                    labelClasses={ w.get('LabelClasses') }
                >
                {
                    w.get('Widgets').map( wi => {
                        // TODO: override the contianer?
                        return renderWidget(wi);
                    })
                }
                </Container>
            );
        }

        return renderWidget(w);
    });
}

export default {
    renderWidgets,
    renderWidget,
    validationForWidget,
};
