
import React from 'react/addons';
const PureRenderMixin = React.addons.PureRenderMixin;
import Immutable from 'immutable';
import Containers from '../containers';
import { Runner } from '../validation';


class BaseInput extends React.Component {

	// TODO: convert this to use immutable
	constructor(props) {
		super(props);
		this.shouldComponentUpdate = PureRenderMixin.shouldComponentUpdate.bind(this);
        this.state = {
			isValidating: false,
			value: this.props.value,
			isPristine: true,
			errors: [],

		    //   _isRequired: false,
		    //   _isValid: true,
		    //   _validationError: '',
		    //   _externalError: null,
		    //   _formSubmitted: false
		};

		// using this as updating this doesn't cause a redraw.
		this.validator = new Runner({
			validators: this.props.validation,
			label: this.props.label,
		});
	}

	componentWillMount() {
		if (this.getFormController()) {
			this.getFormController().attatchToForm(this);
		}

	}

	componentWillUnmount() {
		if (this.getFormController()) {
			this.getFormController().detatchFromForm(this);
		}
	}

	// componentWillReceiveProps(nextProps) {
	// 	if (nextProps.value) {
	// 		this.setState({value: nextProps.value});
	// 	}
	// }

	getFormController() {
		return this.props.formController;
	}

	resetValue() {
		// this.setState({value: this.validator.rawValue});

		// TODO: find dom node and set value
		// TODO: have a set value and get value for dom nodes?
	}

	// TODO: wrap all the setStates into 1
	_setValidating(isValidating) {
		this.setState({isValidating: isValidating});
	}

	_getValidating() {
		return this.state.isValidating;
	}

    /**
     * Starts the validation process by trigger the chain of validators.
     * @param ev The event that is the source trigger for the validation.
     * Returns void.
     */

    _startValidation(val) {
		// Be good to know if the validations are running.
		this._setValidating(true);

		// Run the validations.
		return this.validator.runValidations(val)
		.then( (res) => {
			this._setValidating(false);
			return this._onValidationComplete(res);
		})
		.catch( err => {
			console.log(err, '<<<<<< ERROR in base.jsx');
			this._setValidating(false);
			throw new Error(err);
		});
    }

	_updateFormController() {
		const result = this.validator.getResult();
		if (this.getFormController()) {
			this.getFormController()._onInputUpdate(this, result);
		}

		if (this.props.onInputUpdate) {
			this.props.onInputUpdate(result);
		}
	}


    /**
     * Called when validation has completed and after the _onValidationStart
     * promise has completed.
     * Override this to interpret the validation result in a context specific
     * way.
     *
     * Check result.
     *
     * @param result The validation result.
     * @private
     */
    _onValidationComplete(result) {
		// TODO: remove this and just pass the result to the container.
		this.setState({
			errors: result.errors,
		}, this._updateFormController );

		// Run any of the functions attached to this widget (update cart or conditional logic)
		if (this.props.onChangeFns) {
			setTimeout(() => {
				this.props.onChangeFns.forEach( fn => {
					fn(this.props.viewerIndex, this.props.name, result.value);
				});
			}, 0);
		}

		return result;
    }

	_onSubmit() {
		return this._updateFromDom();
	}


	_onChange(e) {
		const val = e.target.value;

		this.setState({
			'value': val,
			'isPristine': false,
		}, () => {
	        this._startValidation(val);
		});
    }

    // Override me!
    renderComponent() {
        return null;
    }

    render() {
        const Container = Containers.getForLayout(this.props.layout);
		return (
			<Container
				label={ this.props.label }
				description={ this.props.description }
				outerClasses={ this.props.outerClasses }
				labelClasses={ this.props.labelClasses }
				isRequired={ this.validator.getIsRequired() }
				errorMsg={ this.state.errors }
			>

                { this.renderComponent() }

            </Container>
        );
				// errorMsg={ this.props.errorMsg }
    }
}

BaseInput.propTypes = {
    name: React.PropTypes.string.isRequired,
    label: React.PropTypes.string.isRequired,
    description: React.PropTypes.string,
    value: React.PropTypes.any,
	onInputUpdate: React.PropTypes.func,
	formController: React.PropTypes.object,
	onChangeFns: React.PropTypes.array,
	layout: React.PropTypes.string,

	validation: React.PropTypes.array, // also only use of immutable

	// validation: React.PropTypes.instanceOf(Immutable.List), // also only use of immutable
	// validatorContext: React.PropTypes.instanceOf(BaseValidationContext),

	options: React.PropTypes.instanceOf(Immutable.List), // This is the available children or options
	outerClasses: React.PropTypes.string,
	innerClasses: React.PropTypes.string,
	labelClasses: React.PropTypes.string,
// 	errorMsg: React.PropTypes.string,

};

BaseInput.defaultProps = {
    layout: 'single',
};


export default BaseInput;
