
import BaseInput from './base.js';
import classNames from 'classnames';
import Marty from 'marty';


class Donation extends BaseInput {

    constructor(props) {
        super(props);

        let showOther = this.props.options
            .filter( o => this.props.value === o.get('Value') )
            .count() === 0;

        // In the case when nothing is selected, hide the other.
        if ( !this.props.value ) {
            showOther = false;
        }

        this.state = { showOther };
    }

    _onChange(e) {

        const val = e.target.value;
        if (val === 'Other') {
            this.setState({
                showOther: true,
                isPristene: false,
            });
        } else {
            this.setState({
                showOther: false,
                isPristene: false,
            });
    		return this._startValidation(val);
        }

        // Start the cart reloading. Timeout makes sure that the value has updated
        // in other stores.
        if (this.props.onChangeFns) {
            setTimeout(() => {
                this.props.onChangeFns.forEach( fn => {
                   fn(this.props.viewerIndex, this.props.name, val);
                });
            }, 0);
        }
    }

    _onChangeOther(e) {
        const val = e.target.value;
        this.setState({isPristene: false});
		return this._startValidation(val);
    }

	_updateFromDom() {
		let val = null;
        this.props.options.some( (o) => {
			const key = this._keyForChild(o);

            const isChecked = React.findDOMNode(this.refs[key]).checked;
            if (isChecked) {
                val = o.get('Value');
                return true;
            }

			return false;
		});

        if (val === 'Other') {
            val = React.findDOMNode(this.refs.other).value;
        }

		// Handle the null case.
        this.setState({isPristene: false});
		return this._startValidation(val);
	}

	_keyForChild(child) {
		return this.props.name + '___' + child.get('Value') + '___' + child.get('Label');
	}

    renderDonationOptions() {
        let options = this.props.options.map( o => {
            const key = this._keyForChild(o);

            let checked = this.props.value === o.get('Value');
            if ( o.get('Value') === 'Other' ) {
                checked = this.state.showOther;
            }

            return (
					<div key={ key } className='radio'>
                        <input
                            type='radio'
                            ref={ key }
                            id={ key }
                            value={ o.get('Value') }
                            name={ this.props.name }
                            onChange={ this._onChange.bind(this) }
                            defaultChecked={ checked }
                        />
    					<label htmlFor={ key }>{ o.get('Label') }</label>
    				</div>
            );
        });

        return (
            <div className='controls donation-group'>
                { options }
            </div>
        );
    }

    renderOther() {
        if ( !this.state.showOther ) {
            return null;
        }

        const outerClasses = classNames({
            'form-group': true,
            'donation-other-holder': true,
            // 'has-error': this._showError(),
        });

        const name = this.props.name + '_other';

        return (
            <div className={ outerClasses }>
				<label htmlFor={ name }>Other amount:</label>
                <div className='controls'>
                    <input
                        id={ name }
                        className='form-control'
                        ref='other'
                        name={ name }
                        onBlur={ this._onChangeOther.bind(this) }
                        type='text'
                        placeholder='Enter amount'
                        defaultValue={ this.props.value } />
                </div>
            </div>
        );

    }

    render() {
        const outerClasses = classNames({
            'form-group': true,
            // 'has-error': this._showError(),
        });
        const labelClasses = classNames({
            'control-label': true,
            'col-sm-4': false,
        });

        let description;
        if (this.props.description) {
            description = (
                <p className='help-block'>
                    <em>{ this.props.description }</em>
                </p>
            );
        }

        return (
            <div className='donation-show'>
                <div className={outerClasses}>

                    <label className={labelClasses}>
                        { this.props.label }
                        { this.props.isRequired ? <span className='required'>*</span> : '' }
                    </label>

                    { description }

                    { this.renderDonationOptions() }

                    {
                        this.props.errorMsg ? (
                            <span className="help-block">
                                { this.props.errorMsg }
                            </span>
                        ) : ''
                    }

                </div>

                { this.renderOther() }

            </div>
        );
    }
}

export default Marty.createContainer(Donation);
