'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});

var _slicedToArray = (function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i['return']) _i['return'](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError('Invalid attempt to destructure non-iterable instance'); } }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _containers = require('./containers');

var _containers2 = _interopRequireDefault(_containers);

var _widgets = require('./widgets');

var _widgets2 = _interopRequireDefault(_widgets);

function getDisplayWidget(type) {
	var widget = undefined;

	switch (type) {
		case 'tel':
		case 'text':
		case 'email':
		case 'conditionaltext':
			widget = _widgets2['default'].TextInput;
			break;
		case 'textarea':
			widget = _widgets2['default'].TextArea;
			break;
		case 'heading':
			widget = _widgets2['default'].Heading;
			break;
		case 'country':
		case 'dropdown':
			widget = _widgets2['default'].Dropdown;
			break;
		case 'checkbox_group':
			widget = _widgets2['default'].CheckboxSingle;
			break;
		case 'linebreak':
			widget = _widgets2['default'].Linebreak;
			break;
		case 'paragraph':
			widget = _widgets2['default'].Paragraph;
			break;
		case 'radio_group':
			widget = _widgets2['default'].RadioGroup;
			break;
		case 'button_group':
			widget = _widgets2['default'].ButtonGroup;
			break;
		case 'date':
			widget = _widgets2['default'].DateInput;
			break;
		case 'time':
			widget = _widgets2['default'].TimeInput;
			break;
		case 'payment_optional_item':
			widget = _widgets2['default'].PaymentOptional;
			break;
		case 'sponsorship':
			widget = _widgets2['default'].Sponsorship;
			break;
		case 'donation':
			widget = _widgets2['default'].Donation;
			break;
		case 'teams':
			widget = _widgets2['default'].Team;
			break;
		case 'cvv':
			widget = _widgets2['default'].Cvv;
			break;
		case 'terms': // TODO: special case!
		default:
			return null;
	}

	return widget;
}

function validationForWidget(w) {
	var vArray = w.get('Validation');
	if (!vArray) {
		return null;
	}

	return vArray.map(function (v) {
		if (v === 'required') {
			return { name: 'required' };
		}

		if (v === 'email') {
			return { name: 'email' };
		}

		if (/^length/.test(v)) {
			var _v$match = v.match(/^length\[(\d+)\,(\d+)\]/);

			var _v$match2 = _slicedToArray(_v$match, 3);

			var min = _v$match2[1];
			var max = _v$match2[2];

			return { name: 'basicText', notShorter: min, notLonger: max };
		}
	}).filter(function (v) {
		return !!v;
	}).toJS();
}

function renderWidget(w, opts) {
	var name = w.get('Name');
	var type = w.get('Type');
	var Widget = getDisplayWidget(type);
	if (!Widget) {
		return null;
	}

	// A unique key for each viewer/widget so that react knows when to redraw.
	return React.createElement(Widget, {
		key: 'widget_' + name,
		type: type,
		name: name,
		layout: w.get('Layout'),
		label: w.get('Label'),
		description: w.get('Description'),
		placeholder: w.get('Placeholder'),
		options: w.get('Children'),
		validation: validationForWidget(w),
		outerClasses: w.get('OuterClasses'),
		innerClasses: w.get('InnerClasses'),
		labelClasses: w.get('LabelClasses'),

		viewerIndex: opts.viewerIndex /* only used for the team widget at this stage */,
		onChangeFns: opts.onChangeFns,
		formController: opts.formController,
		onInputUpdate: opts.onInputUpdate,
		value: opts.value

	});
}
// validatorContext = {w.get('ValidationContext')}
// value={ this.state.values[name] }
// errorMsg={ this._errorMsgForWidget(w) }
// onInputUpdate={ this._onInputUpdate.bind(this, w) }

// TODO: container recursion?
function renderWidgets(widgets) {
	return widgets.map(function (w) {
		if (w.get('Type') === 'group') {

			// TODO: add default..
			var Container = _containers2['default'].getForLayout(w.get('Layout'));
			// errorMsg={ this._errorMsgForContianer(w.get('Widgets')) }
			// isRequired={ this._isRequiredForContainer(w.get('Widgets')) }
			return React.createElement(
				Container,
				{
					key: w.get('Name'),
					label: w.get('Label'),
					description: w.get('Description'),
					outerClasses: w.get('OuterClasses'),
					labelClasses: w.get('LabelClasses')
				},
				w.get('Widgets').map(function (wi) {
					return renderWidget(wi);
				})
			);
		}

		return renderWidget(w);
	});
}

exports['default'] = {
	renderWidgets: renderWidgets,
	renderWidget: renderWidget,
	validationForWidget: validationForWidget
};
module.exports = exports['default'];

// TODO: override the contianer?